#     Originally created by Arthur-Ervin Avramiea (2020), arthur.avramiea@gmail.com

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

import multiprocessing as mp
import subprocess
from configuration import configuration as conf
import os
import sys

#parallel process worker - initializes params (needs to be done for each process), runs trial
def worker(script_filename,simulation_parameters_filename,params_string,parameter_line_idx,total_params_len,q):
    configuration = conf.get_configuration()
    
    python_command = sys.executable
    
    f=open(os.path.join(configuration[conf.LOGS_FILE_PATH],"log"+str(parameter_line_idx)+".txt"),"w")
    command_array=[python_command,script_filename,simulation_parameters_filename]
    command_array.extend(params_string.split())
    subprocess.call(command_array,stdout=f,stderr=f)
    
    print("processed parameter line: "+str(parameter_line_idx)+"/"+str(total_params_len))

#parallel process listener - prints results to file and to screen
def listener(q):
    '''listens for messages on the q, writes to file. '''
    while 1:
        m = q.get()
        if m == 'kill':
            break
        else:
            print("done")
    
def main(script_filename,simulation_parameters_filename,paramspace_filename):
    configuration = conf.get_configuration()
    
    batch_file_path = os.path.join(configuration[conf.SIMULATION_PARAM_FILES_PATH],paramspace_filename)
    
    if not os.path.exists(configuration[conf.LOGS_FILE_PATH]):
        os.makedirs(configuration[conf.LOGS_FILE_PATH])
    
    manager = mp.Manager()
    q = manager.Queue()

    pool = mp.Pool(mp.cpu_count()-1)

    #listener gets the results from the different files, outputs to screen and prints to file. put listener to work first
    pool.apply_async(listener, (q))

    #fire off workers- which take the trials and distribute them to the processors
    jobs = []
    with open(batch_file_path) as f:
        all_params_strings = f.readlines()
        parameter_line_idx=0
        total_parameters_len = len(all_params_strings)
        for params_string in all_params_strings:
            parameter_line_idx=parameter_line_idx+1
            job = pool.apply_async(worker, (script_filename,simulation_parameters_filename,params_string,parameter_line_idx,total_parameters_len,q))
            jobs.append(job)

    #collect results from the workers through the pool result queue
    for job in jobs:
        job.get()

    #now we are done, kill the listener
    q.put('kill')
    pool.close()

if __name__ == "__main__":
    if len(sys.argv)==4:
        script_filename = sys.argv[1]
        simulation_parameters_filename = sys.argv[2]
        paramspace_filename = sys.argv[3]
        
        main(script_filename,simulation_parameters_filename,paramspace_filename)        
    else:
        print("Missing parameters. Should run as follows: 'python batch_run.py script_filename.py ParametersFile.Parameters Paramspace.Batch' where script_filename.py is the simulation/network generation script, the Parameters file represents the simulation parameters, and the Batch file contains all parameter sets to be run.")