#     Originally created by Arthur-Ervin Avramiea (2020), arthur.avramiea@gmail.com

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

from os.path import isdir, join, isfile, dirname, normpath, basename
from os import listdir,path
import sys
from utils import parameter_utils as p
from collections import OrderedDict

import csv

def csv_to_array(file_path):
    table = []
    with open(file_path,'r') as f:
        reader = csv.reader(f)
    
        for row in reader:
            table.append(row)
    return table

def get_headers_from_table(table,prepend_label_headers,has_header_row=True):

    headers_table = []
    
    
    if has_header_row:
        header_row = table[0]
        start_row_idx = 1
    else:
        start_row_idx = 0
    
    for col_idx in range(1,len(table[0])):
        if has_header_row:
            header_column_title = header_row[col_idx]
            column_header_prepend = prepend_label_headers+'_'+header_column_title
        else:
            column_header_prepend = prepend_label_headers
            
        for i in range(start_row_idx,len(table)):
            headers_table.append(column_header_prepend + '_'+ table[i][0] )
    
    return headers_table

def get_values_from_table(table,has_header_row=True):

    headers_table = []
    
    header_row = table[0]
    
    if has_header_row:
        start_row_idx = 1
    else:
        start_row_idx = 0

    for col_idx in range(1,len(header_row)):
        
        for i in range(start_row_idx,len(table)):
            headers_table.append( table[i][col_idx] )
    
    return headers_table

if len(sys.argv)<2:
    print("Missing path argument. Please provide path for directory where data is located.")
    sys.exit()

dirs_path = sys.argv[1]

onlyfolders = [ f for f in listdir(dirs_path) if isdir(join(dirs_path,f)) ]

string = ''
table_header_created=False
table_header=[]

for f in onlyfolders:
    folder_path = join(dirs_path,f)

    params_file = join(folder_path,"Simulation.Parameters")
    criticality_fitness_file = join(folder_path,"ActivityStatistics.txt")
    
    if isfile(criticality_fitness_file) and isfile(params_file):
        params_from_paramsfile = p.load_params_file(params_file,[])
        
        params_target = OrderedDict()
        
        #copy only those parameters that are of interest
        simulation_parameters_string = params_from_paramsfile[p.SIMULATION_PARAMETERS]
        simulation_parameters_array=map(str.strip,simulation_parameters_string.split(","))
        
        
        for i in range(len(simulation_parameters_array)):
            key = simulation_parameters_array[i]
            params_target[key]=params_from_paramsfile[key]
            
        params_target[p.RUN_COMMAND]=params_from_paramsfile[p.RUN_COMMAND]
        params_target[p.OUTPUT_DIR_PATH_FORMAT]=params_from_paramsfile[p.OUTPUT_DIR_PATH_FORMAT]
        params_target[p.OUTPUT_DIR_PATH_PARAMETERS]=params_from_paramsfile[p.OUTPUT_DIR_PATH_PARAMETERS]
        
        try:
            criticality_fitness_file = p.load_params_file(criticality_fitness_file,[])
            for key in criticality_fitness_file.keys():
                params_target[key]=criticality_fitness_file[key]
    
            data_path = folder_path+"/data/"
            print(data_path)
            print(criticality_fitness_file.keys())
            

	    files_at_path = 'NetworkStatistics.txt'
            network_statistics_params = p.load_params_file(join(folder_path,files_at_path),[])
            for key in network_statistics_params.keys():
                params_target[key]=network_statistics_params[key]
            
            if not table_header_created:
                # this makes sure that I first start with the parameters that differentiate the simulations from one another
                # otherwise the order gets jumbled when I make a list from a keyset
                table_header = simulation_parameters_array
                
                #and then I add the run command
                table_header.append(p.RUN_COMMAND)
                table_header.append(p.OUTPUT_DIR_PATH_FORMAT)
                table_header.append(p.OUTPUT_DIR_PATH_PARAMETERS)
                
                #and then I add the rest
                all_params_to_add = list(params_target.keys())
                for i in range(len(all_params_to_add)):
                    crt_param_to_add = all_params_to_add[i]
                    if crt_param_to_add not in table_header:
                        table_header.append(crt_param_to_add)
                
                string+=";".join(table_header)+"\n"
                table_header_created=True
                
            should_stop = False
            values_array=[]
            for key in table_header:
                if key in params_target.keys():
                    values_array.append(str(params_target[key]))
                else:
                    print("missing key:",key)
                    values_array.append(" ")
            
            if should_stop:
                continue
    
            string+=";".join(values_array)+"\n"
        except Exception as e:
            print("error:",e.message)
    else:
        print("missing files for directory: ",folder_path)

parent_dir = dirname(normpath(dirs_path))
data_dir_name = basename(normpath(dirs_path))

with open(path.join(parent_dir,data_dir_name+"_results.csv"),"w") as f:
    f.write(string)