#     Originally created by Arthur-Ervin Avramiea (2020), arthur.avramiea@gmail.com

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
import sys
import os
from utils import output_utils
from utils import parameter_utils as p
from utils import network_utils
from configuration import configuration as conf
import random
import h5py

configuration = conf.get_configuration()
simulation_param_files_path = configuration[conf.SIMULATION_PARAM_FILES_PATH]

########################################## PARAMETERS LOAD
parameters = p.get_default_parameters()

#override with params passed as arguments
rest_of_arguments = sys.argv[2:]
params_file_path = os.path.join(simulation_param_files_path, sys.argv[1])
params_to_override = p.load_params_file(params_file_path,rest_of_arguments)
for crt_key in params_to_override.keys():
    parameters[crt_key]=params_to_override[crt_key]

#set random seed
random_seed = p.get_random_seed()
parameters[p.RANDOM_SEED]=random_seed
random.seed(parameters[p.RANDOM_SEED])

#initialize output
[output_dir_path,tmp_dir_path] = output_utils.create_simulation_directories(parameters)

print(output_dir_path)

########################################## CREATE OR LOAD NETWORK

#initialize network structure before standalone cpp (code generator cannot handle cpp)
num_neurons=int(parameters[p.NUMBER_NEURONS])
grid_size=parameters[p.GRID_SIZE]

#default all neurons are set to excitatory
types=np.zeros(num_neurons)+1

num_excitatory = int(round(parameters[p.FRACTION_EXCITATORY]*num_neurons))
num_inhibitory = num_neurons-num_excitatory

[xx,yy] = network_utils.get_random_points_placement(grid_size,num_inhibitory)
indices_inhibitory = xx*grid_size+yy

types[indices_inhibitory]=0

from_to = network_utils.get_network(num_neurons,types,parameters)
    
wEE = parameters[p.W_EX_EX]
wEI = parameters[p.W_EX_IN]
wIE = parameters[p.W_IN_EX]
wII = parameters[p.W_IN_IN]

weights = np.zeros(np.shape(from_to)[0])
weights[ np.where( (types[from_to[:,0]]==1)  & (types[from_to[:,1]]==1) )[0]]=wEE
weights[ np.where( (types[from_to[:,0]]==1)  & (types[from_to[:,1]]==0) )[0]]=wEI
weights[ np.where( (types[from_to[:,0]]==0)  & (types[from_to[:,1]]==1) )[0]]=wIE
weights[ np.where( (types[from_to[:,0]]==0)  & (types[from_to[:,1]]==0) )[0]]=wII

output_directory_path = output_utils.get_output_dir_path(parameters)

#first create new empty simulation data hdf5 file and then append to it
f = h5py.File(os.path.join(output_directory_path,"SimulationData.h5"), "w")
f.close()

network_utils.write_network_structure(output_directory_path, types, from_to, weights)

# ############### check if it worked
# [types2,from_to2,weights2] = network_utils.get_network_arrays_from_path(os.path.join(output_directory_path,"SimulationData.h5"))
#  
# wEE = weights[ np.where( (types2[from_to2[:,0]]==1)  & (types2[from_to2[:,1]]==1) )[0]]
# wEI = weights[ np.where( (types2[from_to2[:,0]]==1)  & (types2[from_to2[:,1]]==0) )[0]]
# wIE = weights[ np.where( (types2[from_to2[:,0]]==0)  & (types2[from_to2[:,1]]==1) )[0]]
# wII = weights[ np.where( (types2[from_to2[:,0]]==0)  & (types2[from_to2[:,1]]==0) )[0]]
#  
# print(np.mean(wEE),np.mean(wEI),np.mean(wIE),np.mean(wII))
# print(np.sum(types==types2))
