#     Originally created by Arthur-Ervin Avramiea (2020), arthur.avramiea@gmail.com

#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

from random import random
from collections import OrderedDict

#SIMULATION PARAMETERS
SIMULATION_TYPE = "Simulation Type"
RANDOM_SEED = "Random seed"
SIMULATION_LENGTH = "Simulation Length"
RUN_NUMBER = "Run Number"

#NETWORK PARAMETERS
NUMBER_NEURONS = "Number Neurons"
GRID_SIZE = "Grid size"
EXCITATORY_CONNECTIVITY = "Excitatory Connectivity"
INHIBITORY_CONNECTIVITY = "Inhibitory Connectivity"
FRACTION_EXCITATORY = "fracEx"
W_EX_EX = "wExEx"
W_EX_IN = "wExIn"
W_IN_IN = "wInIn"
W_IN_EX = "wInEx"
EE_GAIN = "eeGain"
EI_GAIN = "eiGain"
IE_GAIN = "ieGain"
II_GAIN = "iiGain"
NETWORK_TO_LOAD_PATH = "Network to load path"
NETWORK_TO_LOAD_PATH_FORMAT = "Network to load path format"
NETWORK_TO_LOAD_PATH_INPUT_PARAMETERS = "Network to load path input parameters"
TORUS_WRAP_AROUND="Torus wrap around"
LOCAL_RANGE_EX = "Local Range Ex"
LOCAL_RANGE_IN = "Local Range In"

#NEURON MODEL PARAMETERS
EXC_DECAY = "excDecay"
INH_DECAY = "inhDecay"
INPUT_DECAY_EX = "inputDecay Ex"
INPUT_DECAY_IN = "inputDecay In"
P_RESET_EXC = "P reset excitatory"
P_RESET_INH = "P reset inhibitory"
BACKGROUND_P_EXC = "Excitatory Background Rate"
BACKGROUND_P_INH = "Inhibitory Background Rate"

EXTERNAL_INPUT = "External input"
STIMULUS_SIZE = "Stimulus size"
WEIGHT_EXTERNAL_INPUT = "Weight external input"

SIMULATION_PARAMETERS = "Simulation parameters"
STANDALONE_MODE = "Standalone mode"
VERBOSE = "Verbose"
OUTPUT = "Output"

OUTPUT_DIR = "Output dir"

OUTPUT_NETWORK = "Output network"
OUTPUT_ACTIVITY = "Output activity"
OUTPUT_DIR_PATH_FORMAT = "Output dir path format"
OUTPUT_DIR_PATH_PARAMETERS = "Output dir path parameters"

RUN_COMMAND="Run command"

SAMPLING_FREQUENCY = "Sampling frequency"

RUN_ANALYSIS_SCRIPT_NAME = "Run analysis script name"

TYPE_EXC=1
TYPE_INH=0

def get_default_parameters():
    default_parameters = OrderedDict()

    default_parameters[STANDALONE_MODE]=True

    default_parameters[OUTPUT] = True
    default_parameters[OUTPUT_NETWORK] = False
    default_parameters[OUTPUT_ACTIVITY] = False
    default_parameters[RUN_NUMBER] = 1
    default_parameters[RUN_ANALYSIS_SCRIPT_NAME] = "run_analysis"
    default_parameters[SAMPLING_FREQUENCY]=1000

    default_parameters[FRACTION_EXCITATORY] = 0.75
    default_parameters[GRID_SIZE] = 50
    default_parameters[NUMBER_NEURONS] = default_parameters[GRID_SIZE]*default_parameters[GRID_SIZE]
    default_parameters[TORUS_WRAP_AROUND] = False

    default_parameters[INPUT_DECAY_EX] = 9
    default_parameters[INPUT_DECAY_IN] = 9 

    default_parameters[EXC_DECAY] = 6
    default_parameters[INH_DECAY] = 12

    default_parameters[P_RESET_EXC] = -2
    default_parameters[P_RESET_INH] = -20
    default_parameters[BACKGROUND_P_EXC] = 0.000001
    default_parameters[BACKGROUND_P_INH] = 0
    default_parameters[W_EX_EX] = (0.0085) # excitatory synaptic weight (voltage)
    default_parameters[W_EX_IN] = (0.0085) # excitatory synaptic weight (voltage)
    default_parameters[W_IN_EX] = (-0.569) # excitatory synaptic weight (voltage)
    default_parameters[W_IN_IN] = (-2) # inhibitory synaptic weight
    default_parameters[EE_GAIN]=1
    default_parameters[EI_GAIN]=1
    default_parameters[IE_GAIN]=1
    default_parameters[II_GAIN]=1

    return default_parameters

#removes additional (unused arguments from string), that is '#' and '*'
unused_arguments = ['#','*']
def cleanup_arguments(arguments_list):
    cleaned_arguments_list = []
    for i in range(len(arguments_list)):
        crt_argument = arguments_list[i]
        if len(crt_argument)==1 and crt_argument in unused_arguments:
            continue
        else:
            cleaned_arguments_list.append(crt_argument)

    return cleaned_arguments_list

def get_param_value(param_value_string):
    property_value = param_value_string.strip()
    
    if is_number(param_value_string):
        property_value = float(param_value_string)
        try:
            property_value_int = int(param_value_string)
            if property_value_int == property_value:
                property_value = property_value_int
        except:
            pass
    elif property_value == "True":
        property_value = True
    elif property_value == "False":
        property_value = False
    
    return property_value

def load_params_file(file_path,rest_of_arguments):
    new_params = load_properties_file(file_path)

    cleaned_arguments = cleanup_arguments(rest_of_arguments)

    if len(rest_of_arguments)>0:
        assert SIMULATION_PARAMETERS in new_params.keys()
        simulation_parameters_vector=new_params[SIMULATION_PARAMETERS].split(",")
        assert len(simulation_parameters_vector)==len(cleaned_arguments)
        for i in range(len(simulation_parameters_vector)):
            new_params[simulation_parameters_vector[i].strip()]=get_param_value(cleaned_arguments[i])
            
    return new_params

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def load_properties_file(file_path):
    new_params = OrderedDict()

    with open(file_path) as f:
        for line in f:
            line_trimmed = line.strip()

            if len(line_trimmed)==0 or line_trimmed[0]=="#" :
                continue

            line_vec=line_trimmed.split("=")

            if len(line_vec)<2:
                continue

            property_name = line_vec[0].strip()
            property_value = line_vec[1].strip()
            new_params[property_name]=get_param_value(property_value)
            
    return new_params

def print_parameters(parameters):
    print("#"*50)
    for crt_key in sorted(parameters.keys()):
        crt_value = parameters[crt_key]
        print(crt_key+"="+str(crt_value))
    print("#"*50)

def get_random_seed():
    return int(round(random()*10000000000000))

def output_params(parameters,path):
    with open(path,"w") as f:
        for crt_key in sorted(parameters.keys()):
            crt_value = parameters[crt_key]
            f.write(crt_key+"="+str(crt_value)+"\n")